#!/bin/perl
############################################
# NIC Monitor
################
# Copyright 2011
################
# Usage:
#
# perl nicmon.pl
#
# Purpose:
#
# The program monitors all known interfaces
# Data appears under node "NIC"
# Based on netstat.pl by Sylvain Gibassier
###########################################
use FindBin;
use lib ("$FindBin::Bin", "$FindBin::Bin/lib/perl", "$FindBin::Bin/../lib/perl");
use Wily::PrintMetric;

use strict;

my $host=`uname -n`;
my $nicmon_log='./logs/nicmon.log';

# define iostat command to be executed
my $netstatCommand='netstat -i';
# hold results
my @nicResults=`$netstatCommand`;

# output of netstat
#Name  Mtu   Network     Address           ZoneID    Ipkts Ierrs    Opkts Oerrs  Coll
#en0   1500  link#2      56.1f.a2.62.ef.2         340485605     0 368803617     0     0
#en0   1500  10.109.132  nydrs1997.us.net.intra   340485605     0 368803617     0     0
#en0   1500  10.109.132  ben-reg.staging.echonet  340485605     0 368803617     0     0
#en0   1500  10.109.132  dumbo2.staging.echonet   340485605     0 368803617     0     0
#en0   1500  10.109.132  rabor2.staging.echonet   340485605     0 368803617     0     0
#lo0   16896 link#1                               591762295     0 591750813     0     0
#lo0   16896 127         loopback                 591762295     0 591750813     0     0
#lo0   16896 ::1                                0 591762295     0 591750813     0     0

chomp($host);
# skip the header row
@nicResults=@nicResults[1..$#nicResults];
# loop through results
foreach my $line (@nicResults){
	#remove EOF
	chomp $line;
	# split the row by column
	my @nicData=split /\s+/, $line;
	# skip row if column 3 doesn't match host or loopback
	if ($nicData[3]=~m/($host)/ or $nicData[3]=~m/loopback/){
		# print metrics
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0] . '|' . $nicData[3],
										name		=> 'Oerrs',
										value		=> $nicData[7],
										);
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0] . '|' . $nicData[3],
										name		=> 'Opkts',
										value		=> $nicData[6],
										);
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0].'|'.$nicData[3],
										name		=> 'Ierrs',
										value		=> $nicData[5],
										);
		Wily::PrintMetric::printMetric (type		=> 'LongCounter',
										resource	=> 'NIC',
										subresource	=> $nicData[0].'|'.$nicData[3],
										name		=> 'Ipkts',
										value		=> int($nicData[4]),
										);
		Wily::PrintMetric::printMetric (type		=> 'StringEvent',
										resource	=> 'NIC',
										subresource	=> $nicData[0].'|'.$nicData[3],
										name		=> 'Network',
										value		=> $nicData[2],
										);
		Wily::PrintMetric::printMetric (type		=> 'StringEvent',
										resource	=> 'NIC',
										subresource	=> $nicData[0].'|'.$nicData[3],
										name		=> 'Mtu',
										value		=> $nicData[1],
										);
	}
}